/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*motifButton.c  */
/*
 *      Author: Janet Anderson
 *      Date:   12-20-90
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 */

#include <stdlib.h>
#include <stdio.h>
/* If not MS Visual C++ or MS Visual C++ is 2010 or later */
#if !defined(_MSC_VER) || _MSC_VER >= 1600
#include <stdint.h>
#endif

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>

#define MAXLINE 250
#define MAXWORDS 25

extern char *optarg; /* needed for getopt() */
extern int optind;   /* needed for getopt() */

static String fallbackResources[] = {
         "motifButton*foreground: black",
         "motifButton*background: #b0c3ca",
/*
         "*pushme.width: 130",
         "*pushme.height: 50",
         "*XmPushButton.fontList: -*-courier-bold-r-*--18-*",
         "*fontList: -*-Helvetica-bold-r-*--24-*",
         "*fontList: -*-courier-bold-r-*--24-*",
         "motifButton*fontList: -*-Helvetica-bold-r-*--18-*",

*/
         "motifButton*fontList: -*-Helvetica-bold-r-*--18-*",
         NULL
};

void doCommand(w, command, cbs)
    Widget w;
    char *command;
    XmPushButtonCallbackStruct *cbs;
{
    char buff[120];
    int l;

    if (command && command[0]!='\0' ) {
        l = strlen(command);
        if (*(command+l-1) == '\n') *(command+l-1) = ' ';
                sprintf(buff,"%s &",command);
                system(buff);
    }
}


void createActionDialog(parent,message1,okCallback,okParm,userParm)
     Widget          parent;
     char           *message1;
     XtCallbackProc okCallback;
     XtPointer      okParm;
     XtPointer      userParm;
{
     static Widget         dialog = 0; /* make it static for reuse */
     XmString              str;
     XmString              str2;

     if (dialog) XtUnmanageChild(dialog);
     if (!dialog) {
          dialog = XmCreateMessageDialog(parent, "Dialog", NULL, 0);
          XtUnmanageChild(XmMessageBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
          XtAddCallback(dialog,XmNcancelCallback,(XtCallbackProc) XtUnmanageChild,NULL);
          XtAddCallback(dialog,XmNokCallback,okCallback,okParm);
     }
     str = XmStringCreateSimple("WarningDialog");
     str2 = XmStringCreateSimple(message1);
     XtVaSetValues(dialog,
          XmNuserData,      userParm,
          XmNdialogType,  XmDIALOG_WARNING,
          XmNdialogTitle, str,
          XmNmessageString, str2,
          NULL);
     XmStringFree(str);
     XmStringFree(str2);
     XtManageChild(dialog);
/*
     XFlush(display);
*/
     return;
}

void i_was_pushed(w, warning, cbs)
    Widget w;
    int warning;
    XmPushButtonCallbackStruct *cbs;
{
    char *command;

    XtVaGetValues(w, XmNuserData, &command, NULL);

    if (warning ) 
        createActionDialog(w,"Are you sure?",(XtCallbackProc)doCommand,command,command);
    else
        doCommand(w, command, cbs);
}

void blinking(pushButton,pblinkTimeoutId)
    XtPointer pushButton;
    XtIntervalId *pblinkTimeoutId;
{
    static int interval=0;
    XtIntervalId blinkTimeoutId;
    Display *display;

    display = XtDisplay(pushButton);
    XRaiseWindow(display,XtWindow(XtParent(pushButton)));
    interval = 10*XtGetMultiClickTime(display);
    blinkTimeoutId = XtAppAddTimeOut(XtWidgetToApplicationContext(pushButton),
                    interval,(XtTimerCallbackProc)blinking, (XtPointer)pushButton);
    XFlush(display);
    return;
}


int main(int argc, char *argv[])
{
    Widget        toplevel ;
    XtAppContext  app;
    void i_was_pushed();
    XmString label;
    Widget pushButton;
    XtIntervalId blinkTimeoutId=0;
    char command[MAXLINE];
    char text[MAXLINE];
    intptr_t    warning=TRUE;
    int c,i;

    toplevel = XtVaAppInitialize(&app, "E-STOP ", NULL, 0,
        &argc, argv, fallbackResources, NULL);

    strcpy(text,"Push Me\0");
    command[0]='\0';

    /* get optional command line parameters */
    while ( (c = getopt(argc, argv, "wc:x:")) != -1)
    {
         switch (c) {
             case 'c':
                 strcpy(command,optarg);
                 break;
             case 'x': 
                 strcpy(text,optarg);
                     break;
             case 'w':
                  warning = FALSE;
                  break;
             default : 
                  fprintf(stderr, "\nusage: %s [Xoptions] [-c command] [-x text] [-w]  \n", argv[0]);
                  fprintf(stderr,"\n\t-c command\tCommand to be executed\n");
                  fprintf(stderr,"\t-x text\t\tText to appear on push button\n");
                  fprintf(stderr,"\t-w \t\tNo warning dialog\n");
                  exit(1);
                  break;
         }
    }

    if (optind < argc) {
        for (i = 0; i < MAXWORDS && optind < argc; optind++, i ++)
            strcat(command,argv[optind]);
    }


    XtVaSetValues(toplevel, XmNtitle, " ", NULL);
    label = XmStringCreateSimple(text); 
    pushButton = XtVaCreateManagedWidget("pushme",
        xmPushButtonWidgetClass, toplevel,
        XmNlabelString, label,
        XmNuserData, command,
        XmNshadowThickness, 5,
        NULL);
    XmStringFree(label);
    XtAddCallback(pushButton, XmNactivateCallback,(XtCallbackProc)i_was_pushed, (XtPointer)warning);

    XtRealizeWidget(toplevel);
    blinking(pushButton,&blinkTimeoutId);
    XtAppMainLoop(app);
    return 0;
}

